import logging

import ckan.plugins as p
import ckan.plugins.toolkit as tk

import ckanext.core_fix.config as conf
import ckanext.core_fix.utils as utils
import ckanext.core_fix.helpers as helper


log = logging.getLogger(__name__)


class CoreFixPlugin(p.SingletonPlugin):
    p.implements(p.IConfigurer)
    p.implements(p.ITemplateHelpers)

    # IConfigurer

    def update_config(self, config_: tk.CKANConfig):
        utils.check_disabled_fixes()
        utils.notify()

        tk.add_template_directory(config_, "templates")
        tk.add_public_directory(config_, "public")
        tk.add_resource("assets", "core_fix")

        if not utils.is_fix_disabled(conf.Fixes.markdown_macro):
            tk.add_template_directory(
                config_, f"templates/{conf.Fixes.markdown_macro.name}/"
            )

    # ITemplateHelpers

    def get_helpers(self):
        return helper.get_helpers()



from __future__ import annotations

import logging

from ckanext.core_fix.config import Fixes, get_disabled_fixes
from ckanext.core_fix.exceptions import CoreFixException

log = logging.getLogger(__name__)


def notify() -> None:
    """Notify about disabled/enabled fixes on startup"""
    for fix in Fixes:
        notify_disabled(fix) if is_fix_disabled(fix) else notify_enabled(fix)


def notify_enabled(fix: Fixes) -> None:
    log.info(f"The `{fix}` fix has been enabled")


def notify_disabled(fix: Fixes) -> None:
    log.info(f"The `{fix}` fix has been disabled")


def check_disabled_fixes() -> None:
    """Check if all fixes listed as disabled are actually exists"""
    available_fixes: list[str] = Fixes._member_names_

    for fix in get_disabled_fixes():
        if is_fix_exist(fix):
            continue

        raise CoreFixException(
            f"The fix `{fix}` doesn't exists. List of available fixes: {available_fixes}"
        )


def is_fix_disabled(fix: Fixes | str) -> bool:
    """Check if fix is disabled"""
    fix_name: str = fix.name if isinstance(fix, Fixes) else fix
    available_fixes: list[str] = Fixes._member_names_

    if fix_name not in available_fixes:
        raise CoreFixException(
            f"The fix `{fix}` doesn't exists. List of available fixes: {available_fixes}"
        )

    return fix_name in get_disabled_fixes()


def is_fix_exist(fix: str) -> bool:
    return fix in Fixes._member_names_



from __future__ import annotations

import logging
from typing import Callable, Any

import ckan.plugins.toolkit as tk

import ckanext.core_fix.config as conf
import ckanext.core_fix.utils as utils


log = logging.getLogger(__name__)


@tk.chained_helper
def dashboard_activity_stream(
    next_func: Callable[..., Any], *args, **kw: Any  # type: ignore
) -> list[dict[str, Any]]:
    """
    Insert `offset` positional argument
    If kw isn't empty, we are started to pass kw args properly

    https://github.com/ckan/ckan/pull/7482
    """

    if not kw:
        args: list[Any] = list(args)
        args.insert(3, 0)

    return next_func(*args, **kw)


def get_helpers():
    helpers: dict[str, Callable[..., Any]] = {
        "dashboard_activity_stream": dashboard_activity_stream,
        "cf_is_fix_disabled": lambda x: utils.is_fix_disabled(x),
    }

    if utils.is_fix_disabled(conf.Fixes.dashboard_activity):
        helpers.pop("dashboard_activity_stream")

    return helpers



from __future__ import annotations

from enum import Enum, auto

import ckan.plugins.toolkit as tk


CONF_DISABLE_FIXES = "ckanext.core_fix.disable_fix"


def get_disabled_fixes() -> list[str]:
    return tk.aslist(tk.config.get(CONF_DISABLE_FIXES, []))


class Fixes(Enum):
    dashboard_activity = auto()
    markdown_macro = auto()
    fontawesome_icons = auto()
    secondary_order = auto()
